<?php

namespace HCM\FPA\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\PercorreHolerite;

class DemonsEletronicoController extends Crud {

    protected $DemonseletronicoService;
    protected $dataSourceName = 'fpa_formholerite';
    protected $dataSourceName_get = 'fpa_formholerite_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }


    public function getFormHolerite(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                if($row['DSFIM']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIM']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceName_get, $rows));
    }
    

    public function setDemonstrativo(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            
            foreach ($rows as &$row) {
                if (array_key_exists('NRFORMHOLERITE', $row)) {
                    if ($row['NRFORMHOLERITE'] === '') {
                        $row['NRFORMHOLERITE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FORMHOLERITE', 1, 12, true);
                    }
                } else {
                    $row['NRFORMHOLERITE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FORMHOLERITE', 1, 12, true);
                }
            }
            if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
 
            if(!array_key_exists('DSFIM', $row)){
                $row['DSFIM'] = null;
            }
            
            // define os campos de descrição de início e fim com base no critério de seleção passado pela row
            if($row['DSORDEM'] === "LISTA_VINCULO"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
            }
            
            if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
                if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                    $row['DSFIM'] = implode(',',$row['DSFIM']);
                }else{
                    if(is_array($row['DSFIM'])){
                        $row['DSFIM'] = null;
                    }
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND") {
                if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                    $row['DSFIM'] = implode(',',$row['DSFIM']);
                }else{
                    if(is_array($row['DSFIM'])){
                        $row['DSFIM'] = null;
                    }
                }
            }
            
            //cria um sequencial manual para o lote
            $row['NRLOTE'] = $this->entityManager->getRepository(Repositories::FPA_FORMHOLERITE)->procuraMaiorLote($this->hcmParameters->getNrorgTrabalho());
            $row['NRLOTE'] = $row['NRLOTE']['MAX(NRLOTE)'] + 1;
            
            $row['NRESTRUTRESPONS'] = isset($row['NRESTRUTRESPONS'])?$row['NRESTRUTRESPONS']:null;
            $row['NRPESSOARESPONS'] = isset($row['NRPESSOARESPONS'])?$row['NRPESSOARESPONS']:null;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function gerarHolerite(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);

            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];

            $preparaHolerite = new PercorreHolerite($dataset, $this->hcmParameters,$this->entityManager);
            $return = $preparaHolerite->percorre();
            
            if(array_key_exists('MENSAGENS',$return)){
                    $row = array();
                    foreach ($return['MENSAGENS'] as $mensagem) {
                        $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                    }
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
            }
            if(array_key_exists('ARQUIVO_ELETRONICO',$return)){
                $arquivo[] = array("NMARQUIVO" => $return['ARQUIVO_ELETRONICO']);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $arquivo));
            }
            
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($return['DSSTATUS'], \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));

        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteDemonstrativo(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 