<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\FPA\Services\IncidenciaEventoService;

class IncidenciaEventoController extends Crud {

    protected $dataSourceNameget = 'fpa_incidevento_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC','P_NREVENTOM');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, IncidenciaEventoService $incidenciaEventoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->incidenciaEventoService = $incidenciaEventoService;
        parent::__construct($dataSourceManager);
    }

    public function getIncidenciaEvento(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameget);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setIncidenciaEvento(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'fpa_incidevento';
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRINCIDEVENTO', $row)) {
                    if ($row['NRINCIDEVENTO'] === '') {
                        $row['NRINCIDEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_INCIDEVENTO', 1, 12, true); 
                    }
                } else {
                    $row['NRINCIDEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_INCIDEVENTO', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function cadAutoIncEvt(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            error_reporting(E_ERROR | E_PARSE);
            $this->dataSourceName = 'fpa_incidevento';
            foreach ($request->getDataSet()->getRows() as &$row) {
                foreach ($row['NRTIPOINCIDE'] as $tipoincide) {
                    foreach ($row['NREVENTOM'] as $eventom) {
                        $this->incidenciaEventoService->cadAutoIncEvt($tipoincide, $eventom, $row['IDINCIDEVENTO']);
                    }
                }
            }
            $response->addNotification(new DTO\Response\Notification('cadAutoDependentes', DTO\Response\Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteIncidenciaEvento(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = 'fpa_incidevento';
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 