<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use HCM\Geral\Repository\ParametroEstruturaRepository;

class ParametroEstruturaService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ParametroEstruturaRepository $parametroEstruturaRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->parametroEstruturaRepository = $parametroEstruturaRepository;
    }
    
    public function deleteParametroEstrutura($dataset) {
        try {
            $this->entityManager->beginTransaction();
            $parametroEstruturah = $this->entityManager->getRepository(Repositories::GPE_PARAMESTRUTH)->findOneBy(array("nrparamestrutm" => $dataset['NRPARAMESTRUTM'], "nrparamestruth" => $dataset['NRPARAMESTRUTH']));
            $this->entityManager->remove($parametroEstruturah);
            $this->entityManager->flush();
            $parametroEstruturahh = $this->entityManager->getRepository(Repositories::GPE_PARAMESTRUTH)->findOneBy(array("nrparamestrutm" => $dataset['NRPARAMESTRUTM']));
            if(empty($parametroEstruturahh)) {
                $parametroEstruturam = $this->entityManager->getRepository(Repositories::GPE_PARAMESTRUTM)->findOneBy(array("nrparamestrutm" => $dataset['NRPARAMESTRUTM'],"nrorg" => $dataset['NRORG']));
                $this->entityManager->remove($parametroEstruturam);
                $this->entityManager->flush();
            }
            $this->entityManager->commit();
        } catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function salvaParametrosEstruturasNaoPreenchidos($nrestruturam, $nrparamfolham){
        //get estruturas sem parametro
        //insert paramEstrutura, mestre e histórico (GPE_PARAMESTRUTM, GPE_PARAMESTRUTH)
        try {
            if($nrestruturam){
                $this->entityManager->beginTransaction();
                
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                
                $dtinivigencia = "01/01/2000 00:00:00";
                $dtfimvigencia = null;
                $nrdiainiperapuracao = 0;
                $nrparamferiasm = null;
                
                $nrparamestrutm = $this->parametroEstruturaRepository->incluiParametrosEstruturaMestre($nrestruturam, $nrorg, $dtinivigencia, $dtfimvigencia);
                $this->parametroEstruturaRepository->incluiParametrosEstruturaHistorico($nrparamestrutm, $nrparamfolham, $nrparamferiasm, $nrorg, null,  $nrdiainiperapuracao);    
                
                $this->entityManager->flush();
                $this->entityManager->commit();
            }
        } catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    public function relacionaParametroFeriasParametroEstrutura($nrestruturam, $nrparamestrutm, $nrparamferiasm){
        if($nrparamferiasm && $nrestruturam){
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $historicoAtual = $this->parametroEstruturaRepository->getHistoricoParametroEstrutura($nrparamestrutm, $nrorg, $dtmescompetenc);
            if(!empty($historicoAtual)){    
                $historicoAtual = $historicoAtual[0];
                $this->parametroEstruturaRepository->incluiParametrosEstruturaHistorico($nrparamestrutm, $historicoAtual['NRPARAMFOLHAM'], $nrparamferiasm, $nrorg, $dtmescompetenc, $historicoAtual['NRDIAINIPERAPURACAO']);
            } else {
                $ultimoHistorico = $this->parametroEstruturaRepository->getUltimoHistorico($nrparamestrutm, $nrorg, $dtmescompetenc);
                if(key($ultimoHistorico) == 0 ){
                    $ultimoHistorico = $ultimoHistorico[0];
                }
                if(!empty($ultimoHistorico['NRPARAMESTRUTM'])){
                    $this->parametroEstruturaRepository->incluiParametrosEstruturaHistorico($nrparamestrutm, $ultimoHistorico['NRPARAMFOLHAM'], $nrparamferiasm, $nrorg, $dtmescompetenc, $ultimoHistorico['NRDIAINIPERAPURACAO']);
                } else {
                    throw new \Exception('Hisórico do parâmetro de estrutura não encontrado');
                }
            }
        }
    }
    
    // public function relacionaParametroFolhaParametroEstrutura($nrestruturam, $nrparamestrutm, $nrparamfolham){
    //     //procura historico na competencia
    //     //se existe: modifica o nrparamfolham pro que foi passado como parametro
    //     //nao exisitndo: cria um historico identico ao anterior porem com o nrparamfolham com o valor passado como parametro
    //     $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
    //     $nrorg = $this->hcmParameters->getNrorgTrabalho();
    //     $paramestruth = $this->entityManager->getRepository(Repositories::GPE_PARAMESTRUTM)->findOneBy(array('nrparamestrutm' => $nrparamestrutm, 'nrorg' => $nrorg, 'dtmescompetenc' => $dtmescompetenc));
    //     if($paramestruth){
    //         $paramestruth->setNrparamestrutm($nrparamestrum);
    //         $this->entityManager->flush();
    //     } else {
    //         if(!empty($ultimoHistorico['NRPARAMESTRUTM'])){
    //             $ultimoHistorico = $this->parametroEstruturaRepository->getUltimoHistorico($nrparamestrum, $nrorg, $dtmescompetenc);
    //             $this->parametroEstruturaRepository->incluiParametrosEstruturaHistorico($nrparamestrutm, $nrparamfolham, $nrorg, $dtmescompetenc, $ultimoHistorico['DTINIVIGENCIA'], $ultimoHistorico['NRDIAINIPERAPURACAO']);
                
    //         } else {
    //             throw new \Exception('Hisórico do parâmetro de estrutura não encontrado');
    //         }
    //     }
    // }
}